from __future__ import annotations

from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.gen.BusinessObjects import TCX_SCConfig
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class ForTypePreferences(TcBaseObj):
    """
    The ForTypePreferences structure contains the information such as Classname, Typename and the property of the
    object for which the Smart Code needs to be generated.
    
    :var className: Contains the Class Name of the object for which the Smart Code needs to be generated.
    
    :var typeName: Contains the Type Name of the object for which the Smart Code    needs to be generated.
    
    :var property: Contains the Property of the object for which the Smart Code needs to be generated.
    """
    className: str = ''
    typeName: str = ''
    property: str = ''


@dataclass
class GetPropertiesForSelectionsOutput(TcBaseObj):
    """
    The GetPropertiesForSelectionsOutput structure contains the information about the properties of selected objects
    for which the Smart Code needs to be generated.
    
    :var clientId: Contains a unique string supplied by the caller. This ID is used to identify return data and partial
    errors associated with this input structure. 
    
    :var containerWiseProperties: Contains the container name (any one of "I", "IR", "IM", "IRM") and the properties
    mapped on to this container in the form of key-value pairs.
    """
    clientId: str = ''
    containerWiseProperties: List[ContainerWiseProperties] = ()


@dataclass
class GetPropertiesForSelectionsResponse(TcBaseObj):
    """
    This structure contains the GetPropertiesForSelectionsOutput object and the ServiceData.
    
    :var output: Contains the information about the properties of selected objects for which the Smart Code needs to be
    generated.
    
    :var serviceData: Contains the successful Object ids, partial error mentioned below.
    o    258000    - If an invalid Root Component is passed, this error is added in the ServiceData.
    """
    output: GetPropertiesForSelectionsOutput = None
    serviceData: ServiceData = None


@dataclass
class GetResultForSelectionsOutput(TcBaseObj):
    """
    The GetResultForSelectionsOutput structure contains the clientID, generated Smart Code and the containerwise
    properties.
    
    :var clientId: Contains a unique string supplied by the caller. This ID is used to identify return data and partial
    errors associated with this input structure.
    
    :var resultStrings: Contains the final generated SmartCode.
    :var containerWiseProperties: Contains the container name (any one of "I", "IR", "IM", "IRM") and the properties
    mapped on to this container in the form of key-value pairs.
    """
    clientId: str = ''
    resultStrings: List[str] = ()
    containerWiseProperties: List[ContainerWiseProperties] = ()


@dataclass
class GetResultForSelectionsResponse(TcBaseObj):
    """
    This structure contains GetResultForSelectionsOutput object and the ServiceData.
    
    :var output: Contains the information about the the generated Smart Code for the given Component's Item ID as per
    the given configuration.
    :var serviceData: The successful Object ids, partial error mentioned below.
    o    258000        - If an invalid Root Component is passed, this error is added in the ServiceData.
    """
    output: GetResultForSelectionsOutput = None
    serviceData: ServiceData = None


@dataclass
class GetValuesForComponentOutput(TcBaseObj):
    """
    The structure GetValuesForComponentsOutput contains the clientID and a vector containing the next avialable Smart
    Code values.
    
    :var clientId: Contains a unique string supplied by the caller. This ID is used to identify return data and partial
    errors associated with this input structure.
    
    :var componentValues: Contains the next valid available Smart Code values for Item ID for the  given  component.
    """
    clientId: str = ''
    componentValues: List[str] = ()


@dataclass
class GetValuesForComponentResponse(TcBaseObj):
    """
    This structure contains GetValuesForComponentsOutput object and the ServiceData.
    
    :var output: Contains the information about the next avialable Smart Code values for Item ID for the given
    component.
    
    :var serviceData: The successful Object ids, partial error mentioned below.
    o    258000    -     If an invalid Root Component is passed, this error is added in the ServiceData.
    """
    output: List[GetValuesForComponentOutput] = ()
    serviceData: ServiceData = None


@dataclass
class ListForerunnersForComponentsOutput(TcBaseObj):
    """
    The structure ListForerunnersForComponentsOutput contains the clientID and SCConfig  configuration object.
    
    :var clientId: Contains a unique string supplied by the caller. This ID is used to identify return data and partial
    errors associated with this input structure
    
    :var forerunningComponents: Contains a list of valid Smart Code configuration objects in Teamcenter.
    """
    clientId: str = ''
    forerunningComponents: List[TCX_SCConfig] = ()


@dataclass
class ListForerunnersForComponentsResponse(TcBaseObj):
    """
    The structure ListForerunnersForComponentsResponse contains the object of ListForerunnersForComponentsOutput and
    the ServiceData.
    
    :var output: Contains information about the components explicitly dependent on each of the given component
    
    :var serviceData: Contains the successful Object ids, partial error mentioned below.
    o    258000        - If an invalid Root Component is passed, this error is added in the ServiceData
    """
    output: List[ListForerunnersForComponentsOutput] = ()
    serviceData: ServiceData = None


@dataclass
class ListObjectsForComponentsOutput(TcBaseObj):
    """
    The structure ListObjectsForComponentsOutput contains the clientID and a vector of TCX_SCConfig  objects.
    
    :var clientId: Contains a unique string supplied by the caller. This ID is used to identify return data and partial
    errors associated with this input structure.
    
    :var componentsList: Contains the list of components explicitly dependent on the given component.
    """
    clientId: str = ''
    componentsList: List[TCX_SCConfig] = ()


@dataclass
class ListObjectsForComponentsResponse(TcBaseObj):
    """
    The structure contains the  ListObjectsForComponentsOutput object and the ServiceData.
    
    :var output: Contains a list of ListObjectsForComponentsoutput objects. Each object represents the components
    explicitly dependent on the given component.
    
    :var serviceData: Contains the successful Object ids, partial error mentioned below.
    o    258000        - If an invalid Root Component is passed, this error is added in the ServiceData.
    """
    output: List[ListObjectsForComponentsOutput] = ()
    serviceData: ServiceData = None


@dataclass
class ListValuesForComponentsOutput(TcBaseObj):
    """
    The structure ListValuesForComponentsOutput contains the clientID and a vector of strings.
    
    :var clientId: Contains a unique string supplied by the caller. This ID is used to identify return data and partial
    errors associated with this input structure.
    :var allowedValues: Contains the list of Smart Code values for Item IDs for the given component as per the Smart
    Code configuration set by the user.
    """
    clientId: str = ''
    allowedValues: List[str] = ()


@dataclass
class ListValuesForComponentsResponse(TcBaseObj):
    """
    The structure ListValuesForComponentsResponse contains a vector of ListValuesForComponentsOutput objects and the
    ServiceData.
    
    :var output: Contains a list of ListValuesForComponentsOutput. Each object contains a list of Smart Code values for
    Item IDs for the given component as mentioned in the Smart Code configuration by the user.
    
    :var serviceData: Contains the successful Object ids, partial error mentioned below.
    o    258000    - If an invalid Root Component is passed, this error is added in the ServiceData.
    """
    output: List[ListValuesForComponentsOutput] = ()
    serviceData: ServiceData = None


@dataclass
class ComponentInfo(TcBaseObj):
    """
    The ComponentInfo structure contains the optional clientId and the TCX_SCConfig configuration object.
    
    :var clientId: Contains  a unique identifier supplied by the caller. This ID is used to 
    track the related objects.
    :var component: Contains a valid Smart code configuration object in Teamcenter.
    """
    clientId: str = ''
    component: TCX_SCConfig = None


@dataclass
class ContainerWiseProperties(TcBaseObj):
    """
    ContainerWiseProperties structure contains the container name (any one of "I", "IR", "IM", "IRM") and the
    properties mapped on to this container in the form of key-value pairs.
    
    :var container: Contains the name of the container, which can be any one of the following -  "I", "IR", "IM" or
    "IRM".
    
    :var properties: This map contains the properties and values of Smart Code configuration object.
    """
    container: str = ''
    properties: PropertiesMap = None


"""
This map contains a key value pairs, where key is the Smart Code configuration property and the value is the user selected value for the property.
"""
CurrentSelectionsMap = Dict[str, str]


"""
This map contains the properties and values of Smart Code configuration object.
"""
PropertiesMap = Dict[str, str]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import ImanVolume
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class VolumeInfoResponse(TcBaseObj):
    """
    This operation returns the list of accessible volumes, default volume and  default local volume for the given user
    and group.
    
    :var accessibleVolumes: A list of accessible volumes for the given user and group combination
    :var defaultVolume: Default volume for  the given  user and group combination
    :var defaultLocalVolume: Default local volume for the given user and group combination
    :var serviceData: The Teamcenter Services structure used to return status of the operation
    """
    accessibleVolumes: List[ImanVolume] = ()
    defaultVolume: ImanVolume = None
    defaultLocalVolume: ImanVolume = None
    serviceData: ServiceData = None



from __future__ import annotations

from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass
from tcsoa.gen.BusinessObjects import ItemRevision


@dataclass
class FileStatus(TcBaseObj):
    """
    Data structure to be used in the response for Simulation Data Monitor Soa services. It contains the information
    about the files found for the traversal path defined in the simulation data monitor config XML
    
    :var heading: Heading to be displayed in the summary section of simulation data monitor.
    :var dstype: Dataset type.
    :var foundCount: Number of valid attached files found
    :var totalCount: Number of components loaded in the simulation Data monitor
    """
    heading: str = ''
    dstype: str = ''
    foundCount: int = 0
    totalCount: int = 0


@dataclass
class MonitorResponse(TcBaseObj):
    """
    Response to be used by the SOA services defined for simulation data monitor.
    
    :var columnHeadings: List of Column headings of monitored columns.
    :var progressHeadings: Data structure containing status for the attached files.
    :var monitoredComponentMap: Monitored component map which contains the list of monitored column.
    :var serviceData: Service Data to be sent with response.
    """
    columnHeadings: List[str] = ()
    progressHeadings: List[FileStatus] = ()
    monitoredComponentMap: MonitoredComponentMap = None
    serviceData: ServiceData = None


@dataclass
class MonitoredColumn(TcBaseObj):
    """
    Data structure to be used in the response of simulation data monitor soa services. It contains the information
    about the attributes.
    
    :var columnHeading: Column Heading under which the attributes of selected components and also the attached files
    will be monitored.
    :var isImage: Boolean to indicate the image
    :var value: Value of the attribute
    """
    columnHeading: str = ''
    isImage: bool = False
    value: str = ''


"""
Map containing list of monitored columns for each component where key is the item revision.
"""
MonitoredComponentMap = Dict[ItemRevision, List[MonitoredColumn]]



from __future__ import annotations

from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class GetLicenseDetailsResponse(TcBaseObj):
    """
    LicenseDetailsResponse structure represents all the details of an ADA ADA_License business objects in  licDetails
    and operation status in serviceData.
    
    :var licDetails: Structures containing the properties of the given licenseADA_License business objects.
    :var serviceData: ServiceDataContains partial errors encountered while fetching properties for ADA_License business
    objects. The ADA_License business objects for which properties are fetched successfully are added to the Plan list.
    """
    licDetails: List[LicenseDetails] = ()
    serviceData: ServiceData = None


@dataclass
class LicenseDetails(TcBaseObj):
    """
    LicenseDetails structure represents all the details of an ADA License object.
    
    :var licenseType: The type of ADA License object. The type can be "ITAR_License", "Exclude_License", or
    "IP_License". The license type of existing licenses cannot be modified. An error will be thrown if the value of
    licenseType does not match the type of the license being modified.
    :var licenseId: The unique ID of the license. This is string with a maximum of 128 bytes.
    :var licenseCategory: The category property of an ADA license. This is an optional parameter of string type 128
    bytes in size. The value specified for licenseCategory parameter must match one of the values in the LOV associated
    with Category property on the specifc ADA license type.This is an optional parameter and may be left as a blank
    string ("").
    :var expiryDate: The expiry date for the license, after which the license cannot be attached to WorkspaceObjects
    and ceases to grant the access to users/groups listed on the license. The value specified for expiryDate should be
    greater than or equal to current date, unless the value is same as current value on an existing license. A NULL
    date specifies the lincense will never expire.
    :var lockDate: The freeze date for the license, after which the license cannot be attached to WorkspaceObjects. The
    value specified for this parameter should be greater than the current date and should be less than the value
    specified for expiryDate parameter, unless the value is same as current value on an existing license. A NULL date
    specifies the license is not locked.
    :var licenseReason: A reason for the license to exist. This parameter can be a maxiumum of 128 bytes. This is an
    optional parameter and may be left as a blank string ("").
    :var qualifyingCfr: The qualifying Code of Federal Regulations (CFR) for ITAR licenses. This is not applicable for
    IP and Exclude licenses. This parameter can be a maxiumum of 80 bytes. This is an optional parameter and  may be
    left as a blank string ("").
    :var users: The list of users associated with a license identified by licenseId. When the license is attached to a
    classified WorkspaceObject, the users listed on the license will get access to the WorkspaceObject, based on the
    access rules.
    :var groups: List of names of groups associated with the license identified by licenseId. For sub-groups, the full
    names should be specified. When the license is attached to a classified WorkspaceObject, the users from the
    groups/sub-groups listed on the license will get access to the workspace object, based on the access rules. This
    parameter takes an array of group name strings of upto 256 bytes in size.
    """
    licenseType: str = ''
    licenseId: str = ''
    licenseCategory: str = ''
    expiryDate: datetime = None
    lockDate: datetime = None
    licenseReason: str = ''
    qualifyingCfr: str = ''
    users: List[str] = ()
    groups: List[str] = ()



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BOMLine
from enum import Enum
from typing import List
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class VariantCondInfo(TcBaseObj):
    """
    Contains the information related to classic variant condition.
    
    :var optionName: Refers to the classic variant option name.
    :var itemId: Refers to item id where classic variant option is defined.
    :var joinOperator: Refers to 'VariantOperator'.  Legal values are: 'AND', 'OR', 'OPEN_BRACKET' and 'CLOSE_BRACKET'.
    :var compOperator: Refers to 'ComparisonOperator'.  Legal values are: 'EQ', 'NEQ', 'LT', 'GT', 'GTEQ' and 'LTEQ'.
    :var value: Refers to classic variant option value.
    """
    optionName: str = ''
    itemId: str = ''
    joinOperator: VariantOperator = None
    compOperator: ComparisonOperator = None
    value: str = ''


@dataclass
class CreateOrUpdateVariantCondInput(TcBaseObj):
    """
    Contains the input for create/update variant condition associated with a BOMLine object.
    
    :var clientId: Identifier that helps the client to track the object(s) created.
    :var operation: Refers to 'Operation' which refers to the operation type. Legal values are: 'Create' and 'Update'.
    :var bomLine: Refers to BOMLine object on which variant condition has been defined.
    :var variantCondInfo: Refers to a list of 'VariantCondInfo' struct, and contains the information needed to
    create/update a variant condition.
    """
    clientId: str = ''
    operation: Operation = None
    bomLine: BOMLine = None
    variantCondInfo: List[VariantCondInfo] = ()


class ComparisonOperator(Enum):
    """
    Contains comparison operator. Legal values are:' EQ, NEQ, LT, GT, GTEQ' and 'LTEQ'.
    """
    EQ = 'EQ'
    NEQ = 'NEQ'
    LT = 'LT'
    GT = 'GT'
    GTEQ = 'GTEQ'
    LTEQ = 'LTEQ'


class Operation(Enum):
    """
    Contains the operation type. Legal values are: 'Create' and 'Update'.
    """
    Create = 'Create'
    Update = 'Update'


class VariantOperator(Enum):
    """
    Contains the join operator type. Legal values are: 'AND', 'OR, OPEN_BRACKET' and 'CLOSE_BRACKET'.
    """
    AND = 'AND'
    OR = 'OR'
    OPEN_BRACKET = 'OPEN_BRACKET'
    CLOSE_BRACKET = 'CLOSE_BRACKET'



from __future__ import annotations

from tcsoa.gen.Internal.Security._2017_12.AwLicensing import LicenseInput, LicensesWithTypesResponse
from tcsoa.utils import TcService


class AwLicensingService(TcService):

    @classmethod
    def getLicensesWithTypes(cls, licenseInputs: LicenseInput) -> LicensesWithTypesResponse:
        """
        This operation retrieves the Licenses attached to the given workspace objects . In case of multiple objects,
        only the licenses that are attached to all of the objects will be returned . It also retrieves level or
        structure information in case of ActiveWorkspace Content context. In ActiveWorkspace Content context, if the
        selected object do not have additional child objects then level or structure information are not be returned.
        """
        return cls.execute_soa_method(
            method_name='getLicensesWithTypes',
            library='Internal-Security',
            service_date='2017_12',
            service_name='AwLicensing',
            params={'licenseInputs': licenseInputs},
            response_cls=LicensesWithTypesResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from tcsoa.gen.Manufacturing._2008_12.IPAManagement import IPAManagementSaveSearchResultInput, IPAManagementGetFilteredIPAResponse, IPAManagementSaveSearchResultResponse, IPAManagementGenerateSearchScopeResponse
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcService


class IPAManagementService(TcService):

    @classmethod
    def saveSearchResult(cls, input: List[IPAManagementSaveSearchResultInput]) -> IPAManagementSaveSearchResultResponse:
        """
        Saves the search result in a new/updated structure.
        """
        return cls.execute_soa_method(
            method_name='saveSearchResult',
            library='Manufacturing',
            service_date='2008_12',
            service_name='IPAManagement',
            params={'input': input},
            response_cls=IPAManagementSaveSearchResultResponse,
        )

    @classmethod
    def deletefilteredIPA(cls, processes: List[BusinessObject]) -> ServiceData:
        """
        Deletes the filteredIPA structure from the process.
        """
        return cls.execute_soa_method(
            method_name='deletefilteredIPA',
            library='Manufacturing',
            service_date='2008_12',
            service_name='IPAManagement',
            params={'processes': processes},
            response_cls=ServiceData,
        )

    @classmethod
    def generateSearchScope(cls, processes: List[BusinessObject]) -> IPAManagementGenerateSearchScopeResponse:
        """
        find the IPA under the given process (for each process) and retrives the bomlines from under it.
        """
        return cls.execute_soa_method(
            method_name='generateSearchScope',
            library='Manufacturing',
            service_date='2008_12',
            service_name='IPAManagement',
            params={'processes': processes},
            response_cls=IPAManagementGenerateSearchScopeResponse,
        )

    @classmethod
    def getFilteredIPA(cls, processes: List[BusinessObject]) -> IPAManagementGetFilteredIPAResponse:
        """
        Return the filteredIPA structure from the process.
        """
        return cls.execute_soa_method(
            method_name='getFilteredIPA',
            library='Manufacturing',
            service_date='2008_12',
            service_name='IPAManagement',
            params={'processes': processes},
            response_cls=IPAManagementGetFilteredIPAResponse,
        )



from __future__ import annotations

from tcsoa.gen.Internal.AWS2._2015_10.FullTextSearch import Awp0FullTextSavedSearchInputObject2, Awp0FullTextSavedSearchResponse, Awp0FullTextSavedSearchInputObject
from tcsoa.gen.BusinessObjects import BusinessObject
from tcsoa.gen.Internal.AWS2._2015_10.DataManagement import DatasetTypesWithDefaultRelation
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcService


class FullTextSearchService(TcService):

    @classmethod
    def modifyFullTextSavedSearch(cls, inputs: List[Awp0FullTextSavedSearchInputObject2]) -> Awp0FullTextSavedSearchResponse:
        """
        This operation modifies existing Awp0FullTextSavedSearch objects.
        """
        return cls.execute_soa_method(
            method_name='modifyFullTextSavedSearch',
            library='Internal-AWS2',
            service_date='2015_10',
            service_name='FullTextSearch',
            params={'inputs': inputs},
            response_cls=Awp0FullTextSavedSearchResponse,
        )

    @classmethod
    def createFullTextSavedSearch(cls, inputs: List[Awp0FullTextSavedSearchInputObject]) -> Awp0FullTextSavedSearchResponse:
        """
        This operation creates Awp0FullTextSavedSearch objects. Awp0FullTextSavedSearch objects are used to store
        information about a saved search such as search name, search string, search filters, etc.
        """
        return cls.execute_soa_method(
            method_name='createFullTextSavedSearch',
            library='Internal-AWS2',
            service_date='2015_10',
            service_name='FullTextSearch',
            params={'inputs': inputs},
            response_cls=Awp0FullTextSavedSearchResponse,
        )

    @classmethod
    def deleteFullTextSavedSearch(cls, objects: List[BusinessObject]) -> ServiceData:
        """
        This operation deletes saved searches. The saved search is unpinned before deleted.
        """
        return cls.execute_soa_method(
            method_name='deleteFullTextSavedSearch',
            library='Internal-AWS2',
            service_date='2015_10',
            service_name='FullTextSearch',
            params={'objects': objects},
            response_cls=ServiceData,
        )


class DataManagementService(TcService):

    @classmethod
    def getDatasetTypesWithDefaultRelation(cls, parent: BusinessObject, fileExtensions: List[str]) -> DatasetTypesWithDefaultRelation:
        """
        This operation returns the dataset type and reference information for a set of file extensions. This operation
        also returns the default paste relation for each Dataset type returned for the given input parent. Named
        references are Teamcenter objects that related to a specific data file.
        
        For each file extension, it is possible that it belongs to multiple dataset types. For such cases, all matching
        dataset types will be returned using the file extension as the key in the
        GetDatasetTypesWithDefaultRelationOutput structure.
        
        The order of file extension in the GetDatasetTypesWithDefaultRelationOutput structure may be different than the
        order of file extension input. This operation inserts file extensions that match the default dataset type
        defined in AE_default_dataset_type preference at the beginning of the list.
        
        This operation uses the TC_Dataset_Import_Exclude_Wildcard preference to determine if wildcard may be used in
        file extension input. If the preference is set and file extension is set to asterisk, this operation will
        return all data set types that allow wildcards in its name reference in Teamcenter.
        
        Default paste relation for each Dataset type returned is determined by evaluating preferences
        __default_relation and/or _default_relation. The preference __default_relation takes precedence over
        _default_relation.
        
        Details about these four preferences can be found in Preferences and Environment (Variables Reference
        Configuration preferences, under Data management preferences).
        """
        return cls.execute_soa_method(
            method_name='getDatasetTypesWithDefaultRelation',
            library='Internal-AWS2',
            service_date='2015_10',
            service_name='DataManagement',
            params={'parent': parent, 'fileExtensions': fileExtensions},
            response_cls=DatasetTypesWithDefaultRelation,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from tcsoa.gen.Classification._2016_03.Classification import SearchClassesExtendedResponse, ConvertValuesResponse, GetChildrenExtendedResponse, ValueConversionInput
from typing import List
from tcsoa.gen.Classification._2007_01.Classification import SearchAttribute
from tcsoa.utils import TcService


class ClassificationService(TcService):

    @classmethod
    def convertValues(cls, valueConversionInputs: List[ValueConversionInput]) -> ConvertValuesResponse:
        """
        This operation converts the input value using provided input and output unit.
        
        The conversion happens only when the output format represents a numerical format. For String and KeyLOV(Stxt)
        formats, no conversion will occur, and the input string will be returned as is.
        
        Use cases:
        When creating classification classes, you can define whether a class contains only metric ICOs, only nonmetric
        ICOs, or both. If the classification administrator specifies that a class can contain both, you can search for
        an object using either of the unit systems you define, and the search mechanism finds a match, regardless of
        the unit in which the object is stored. For example, if you search for a bolt with a width of 5/8th inches, the
        classification search mechanism finds a bolt that is stored with a width of 1.6 centimeters. This operation
        supplies this conversion mechanism.
        """
        return cls.execute_soa_method(
            method_name='convertValues',
            library='Classification',
            service_date='2016_03',
            service_name='Classification',
            params={'valueConversionInputs': valueConversionInputs},
            response_cls=ConvertValuesResponse,
        )

    @classmethod
    def searchClassesExtended(cls, searchCriterias: List[SearchAttribute], sortAttributes: List[int], filterForWriteAccess: bool, options: int) -> SearchClassesExtendedResponse:
        """
        Finds the classification classes based on provided search criteria and provides detailed information about
        those classes. The user can search using a search expression on attributes of the class (class ID, name, used
        attribute etc.). For example, the user shall be able to search all the classes whose name begins with a
        particular set of characters and where the class ID matches certain pattern. The order of search results can
        also be sorted on various criteria. 
        
        In comparison to 'searchForClasses' it contains extended information about the class itself and also extended
        information about its parent classes.
        
        Use cases:
        The user needs to search for classification classes using a search criterion based on various attributes of a
        class. The search criterion can be based on one or more attributes.
        """
        return cls.execute_soa_method(
            method_name='searchClassesExtended',
            library='Classification',
            service_date='2016_03',
            service_name='Classification',
            params={'searchCriterias': searchCriterias, 'sortAttributes': sortAttributes, 'filterForWriteAccess': filterForWriteAccess, 'options': options},
            response_cls=SearchClassesExtendedResponse,
        )

    @classmethod
    def getChildrenExtended(cls, classObjects: List[BusinessObject], filterForWriteAccess: bool, options: int) -> GetChildrenExtendedResponse:
        """
        Gets the detailed information about immediate children in the classification hierarchy for given group(s) or
        class(es). It also contains all detailed information about the parent classes within the hierarchy. 
        
        In comparison to 'getChildren' it contains extended information about the class itself and also extended
        information about its parent classes.
        
        Use cases:
        User wants to get the details of the class "Tools and Components" and all the classes under it, to render the
        entire hierarchy.
        """
        return cls.execute_soa_method(
            method_name='getChildrenExtended',
            library='Classification',
            service_date='2016_03',
            service_name='Classification',
            params={'classObjects': classObjects, 'filterForWriteAccess': filterForWriteAccess, 'options': options},
            response_cls=GetChildrenExtendedResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import IDCWindow
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class CreateIDCWindowOutput(TcBaseObj):
    """
    Contains IDCWindow created and index to map IDCWindow to source BOMLine OR ItemRevision.
    
    :var inputIndex: Integer pointing to input BOMLine or ItemRevision.
    :var idcWindow: The IDCWindow created for input BOMLine or ItemRevision.
    """
    inputIndex: int = 0
    idcWindow: IDCWindow = None


@dataclass
class CreateIDCWindowResponse(TcBaseObj):
    """
    Contains 'CreateIDCWindowOutput' containing created IDCWindow and index to map with source BOMLine OR ItemRevision.
    Response also contains 'ServiceData'.
    
    :var output: 'FindTargetsOutput' containing created IDCWindow for input BOMLine or ItemRevision and integer index
    to map IDCWindow to input source.
    :var serviceData: 'ServiceData' with created objects containing IDCWindow in created list and partial errors.
    """
    output: List[CreateIDCWindowOutput] = ()
    serviceData: ServiceData = None



from __future__ import annotations

from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetTransientFileTicketsResponse(TcBaseObj):
    """
    Holds the response returned from the 'getTransientFileTicketsForUpload' operation.
    
    :var transientFileTicketInfos: The requested transient files ticket information.
    :var serviceData: This contains the status of the operation.
    """
    transientFileTicketInfos: List[TransientFileTicketInfo] = ()
    serviceData: ServiceData = None


@dataclass
class TransientFileInfo(TcBaseObj):
    """
    Holds the basic information for a file to be uploaded.
    
    :var fileName: The name of the file. Path of the file should not be supplied.
    :var isBinary: True if the file is of binary type, false for text files.
    :var deleteFlag: True if the file should be deleted from temporary storage after it is read.
    """
    fileName: str = ''
    isBinary: bool = False
    deleteFlag: bool = False


@dataclass
class TransientFileTicketInfo(TcBaseObj):
    """
    Holds the file information with a ticket added.
    
    :var transientFileInfo: The unique identifier of the file to be uploaded.
    :var ticket: Holds the basic information for a file to be uploaded.
    """
    transientFileInfo: TransientFileInfo = None
    ticket: str = ''



from __future__ import annotations

from tcsoa.gen.BusinessObjects import User, ImanRelation, Fnd0Qualification, Discipline, ResourcePool
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class FilterCriteria(TcBaseObj):
    """
    The filter criteria for retrieving the list of matching Users.
    
    :var disciplines: A list of Discipline object for which User needs to be filtered.
    :var grouprole: A list of ResourcePool objects from which the Group and Role information is taken for filtering the
    User.
    :var qualifications: List of  criteria specifying detailed information of the Fnd0Qualification assignments for
    which the user needs to be filtered.
    """
    disciplines: List[Discipline] = ()
    grouprole: List[ResourcePool] = ()
    qualifications: List[QualificationInfo] = ()


@dataclass
class FilteredUser(TcBaseObj):
    """
    Individual User objects which match the input filter criteria of Discipline, Group, Role and Fnd0Qualification.
    
    :var user: The User matching the filter criteria.
    :var discipline: The Discipline that the user is a part of.
    :var groupRole: The Group/Role that the user is a part of.
    :var qualificationRel: The Fnd0Qualification which is assigned to the User.
    """
    user: User = None
    discipline: Discipline = None
    groupRole: ResourcePool = None
    qualificationRel: ImanRelation = None


@dataclass
class FilteredUsersInfo(TcBaseObj):
    """
    The structure which contains list of User objects which match the input filter criteria of Discipline, Group, Role
    and Fnd0Qualification. If there are no User objects found which match the input filter criteria, an empty response
    is returned.
    
    :var filtereduserinfo: A list containing information about the filtered Users.
    :var servicedata: Service Data
    """
    filtereduserinfo: List[FilteredUser] = ()
    servicedata: ServiceData = None


@dataclass
class QualificationInfo(TcBaseObj):
    """
    List of  criteria specifying detailed information of the Fnd0Qualification assignments for which the user needs to
    be filtered
    
    
    :var qualification: The Fnd0Qualification object for which User needs to be filtered. 
    :var level: Level of Fnd0Qualification at which Users needs to be filtered.Level is defined on Fnd0Qualification
    object by the Administrator who creates the Fnd0Qualification objects.
    (for example "Advanced" , "Beginner").
    
    :var isfindAlternates: If true the operation returns all the Users who are on the same level as specified in the
    input and also the Users who are on a higher level than the level specified in the input.
    If false the operation returns only the Users who are on the exact level as specified in the input.
    """
    qualification: Fnd0Qualification = None
    level: str = ''
    isfindAlternates: bool = False



from __future__ import annotations

from typing import Dict, List
from tcsoa.gen.BusinessObjects import WorkspaceObject, PSBOMView, MEActivity, MENXObject
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class MEActivityFolderInfo(TcBaseObj):
    """
    TODO
    
    :var activity: The MEActivity object tag in
    cases of update
    :var name: Name of the activity folder that
    needs to created/updated.
    :var description: Description of the acivity folder
    that needs to created or updated.
    :var type: Type of the activity folder that
    needs to be created.
    :var toolInfo: toolInfo
    :var contents: Objects that need to be inserted
    into the activity folder as
    contents.
    :var parentActivities: Parent activity folder into which
    this folder needs to inserted as
    a content.
    :var predecessors: MEActivity folder that need to
    added as predecessors.
    :var attributes: A vector of NameValueStruct  that
    has the name value pairs.
    Through this act_location, time
    and start_time can be set for the
    folder.
    :var complete: Whether the given information
    completely represents the folder
    to be updated.
    This is flag is applicable only
    for completely replacing tools,
    predecessors and contents.
    """
    activity: MEActivity = None
    name: str = ''
    description: str = ''
    type: str = ''
    toolInfo: ActivityToolInfo = None
    contents: List[WorkspaceObject] = ()
    parentActivities: List[MEActivity] = ()
    predecessors: List[MEActivity] = ()
    attributes: List[NameValueStruct] = ()
    complete: bool = False


@dataclass
class MENXObjectInfo(TcBaseObj):
    """
    TODO
    
    :var menxObject: The MENXObject object tag in cases of update
    :var name: Name of the MENXObject that needs to created or updated.
    :var description: Description of the MENXObject that needs to created or updated.
    :var type: Type of the MENXObject that needs to created.
    :var subType: Sub-Type of the MENXObject that needs to created.
    :var attributes: A vector of NameValueStruct  that has the name value pairs.
    Through this the attributes of the MENXObject like double_attrs,
    double_keys, int_attrs, int_keys, string_attrs, string_keys can be set.
    """
    menxObject: MENXObject = None
    name: str = ''
    description: str = ''
    type: str = ''
    subType: str = ''
    attributes: List[NameValueStruct] = ()


@dataclass
class NameValueStruct(TcBaseObj):
    """
    TODO
    
    :var name: Title of the attribute that
    needs to be set
    :var values: Values of the attribute to be set
    """
    name: str = ''
    values: List[str] = ()


@dataclass
class ActivityToolInfo(TcBaseObj):
    """
    TODO
    
    :var processBV: The context bom view
    :var toolOccThreadChains: The context bom view
    """
    processBV: PSBOMView = None
    toolOccThreadChains: OccThreadChainsSet = None


@dataclass
class CreateOrUpdateMEActivityFolderResponse(TcBaseObj):
    """
    TODO
    
    :var successfullyProcessedMEAct: successfullyProcessedMEAct
    :var serviceData: serviceData
    """
    successfullyProcessedMEAct: SuccessFullyProcessedMapMEAct = None
    serviceData: ServiceData = None


@dataclass
class CreateOrUpdateMENXObjectResponse(TcBaseObj):
    """
    TODO
    
    :var successfullyProcessedMENXObj: successfullyProcessedMENXObj
    :var serviceData: serviceData
    """
    successfullyProcessedMENXObj: SuccessFullyProcessedMapMENXObj = None
    serviceData: ServiceData = None


"""
OccThreadChainsSet
"""
OccThreadChainsSet = Dict[int, List[str]]


"""
SuccessFullyProcessedMapMEAct
"""
SuccessFullyProcessedMapMEAct = Dict[int, MEActivity]


"""
SuccessFullyProcessedMapMENXObj
"""
SuccessFullyProcessedMapMENXObj = Dict[int, MENXObject]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import Fnd0ProductSessionData, Item, BusinessObject, AssemblyArrangement, RevisionRule, Folder, ItemRevision, CFMOverrideEntry, ImanFile, BOMWindow
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class NamedReferenceInfo(TcBaseObj):
    """
    'NamedReferenceInfo' structure contains the reference object corresponding to the named reference. If the
    referenced object is ImanFile then the FMS file ticket for it is returned.
    
    :var namedReferenceType: The type of reference object.
    :var namedReferenceName: The 'NamedReference' name of the object.
    :var referenceObject: The object reference corresponding to the named reference.
    :var fileTicket: The FMS ticket used to retrieve the file in the cases where 'referenceObject' is a file.
    """
    namedReferenceType: str = ''
    namedReferenceName: str = ''
    referenceObject: BusinessObject = None
    fileTicket: str = ''


@dataclass
class OpenSavedSessionFilter(TcBaseObj):
    """
    'OpenSavedSessionFilter' enables the client application to control the output returned during opening a session.
    
    :var relAndTypesFilter: A list of type filter to be applied on the objects that are associated to the session.
    :var productStructureFilter: The filter to control the product structures returned by the operation. This structure
    can be empty in which case the default values as mentioned in the 'ProductStructureFilter' description will be used.
    :var productSessionDataFilter: Product Session Data Filter
    """
    relAndTypesFilter: List[RelationAndTypesFilter] = ()
    productStructureFilter: ProductStructureFilter = None
    productSessionDataFilter: ProductSessionDataFilter = None


@dataclass
class OpenSessionOutput(TcBaseObj):
    """
    'OpenSessionOutput' contains the launched session object,  the associated product structures and objects.
    
    :var sessionObject: The session Fnd0AppSession object that was opened from the client.
    :var sessionProductStructures: A list of associated product structures that were associated to the session.
    Typically, it contains the BOMWindow object.
    :var relatedObjectInfos: A list of 'RelatedObjectInfo' that contains information about the objects that were
    related to the session object.
    :var productSessionDataObjects: A list of Fnd0ProductSessionData that are associated to the session.
    """
    sessionObject: BusinessObject = None
    sessionProductStructures: List[SessionProductStructureInfo] = ()
    relatedObjectInfos: List[RelatedObjectInfo] = ()
    productSessionDataObjects: List[ProductSessionDataObjectInfo] = ()


@dataclass
class OpenSessionResponse(TcBaseObj):
    """
    'OpenSessionResponse' contains a list of 'OpenSessionOutput' which has the objects and product structures
    associated to the session. Stable Id that identifies the associated objects and product structures are also
    returned which could be used for further references to update the session.
    
    :var sessionOutputs: A list of objects and product structures that were associated to the session.
    :var serviceData: 'ServiceData' that contains the partial errors if any.
    """
    sessionOutputs: List[OpenSessionOutput] = ()
    serviceData: ServiceData = None


@dataclass
class OverrideInfo(TcBaseObj):
    """
    This contains information about the override RevisionRule Entry.
    
    :var ruleEntry: Refers to the CFMOverrideEntry of RevisionRule object.
    :var folder: A Folder containing ItemRevision that are to be considered while evaluating the RevisionRule.
    """
    ruleEntry: CFMOverrideEntry = None
    folder: Folder = None


@dataclass
class ProductSessionDataAttachInfo(TcBaseObj):
    """
    'ProductSessionDataAttachInfo' is a structure which enables to create or update an object of type
    Fnd0ProductSessionData and attach it to the session.
    
    :var clientId: The input 'clientId' sent by the client application which will enable the client to map the output
    with the input.
    :var sessionData: Fnd0ProductSessionData that has to be created or updated and attached to the session object.
    """
    clientId: str = ''
    sessionData: CreateOrUpdateObjectInfo = None


@dataclass
class ProductSessionDataFilter(TcBaseObj):
    """
    'ProductSessionDataFilter' is a structure in which the application can specify the Fnd0ProductSessionData types
    that are associated to the session. The application can either specify the Fnd0ProductSessionData by their stable
    Ids or by their type names.
    
    :var stableIds:  A list of stable Ids that identifies the association of Fnd0ProductSessionData to the session.
    :var productSessionDataTypeNames:  A list of Fnd0ProductSessionData type names. The instances of the given type
    which are associated to the session that are to be retrieved.
    """
    stableIds: List[str] = ()
    productSessionDataTypeNames: List[str] = ()


@dataclass
class ProductSessionDataObjectInfo(TcBaseObj):
    """
    'ProductSessionDataObjectInfo' has the information about Fnd0ProductSessionData that are associated to the session.
    
    :var stableId: The identifier that enables the client to identify the association of the opened object with the
    session.
    :var productSessionDataObject: Fnd0ProductSessionData object associated to the session.
    """
    stableId: str = ''
    productSessionDataObject: Fnd0ProductSessionData = None


@dataclass
class ProductStructureFilter(TcBaseObj):
    """
    This structure enables the client to control how and what product structures are to returned in the response.
    
    :var productStructure: A string used to identify what product structure associated to the session are to be
    returned. 
    Valid values are:
     "'AllProducts'" &ndash; Return all product structures associated to the session.
    
    "'UseList'"-Returned the product structures that are identified by the ids in  'relativeToSessionStableIds'.
    
    "'NoProducts'" &ndash; Don&rsquo;t return any product structures.
    Note: Default value is 'AllProducts'.
    :var outputFilters: A map('string,string') that specifies various options that need to be applied when returning
    the session product structure contents. Valid values are
    "'RecipeObjectsOnly = true'" -  When true, the product structure BOMWindow is not returned and instead the recipe
    object of the product structure which is usually a StructureContext that can be opened outside the session context
    is returned. Default value is false.
    "'WantStaticStructureFileTicket = true'" -  When true, the file ticket of the static product structure file is
    returned. Default value is false.
    :var relativeToSessionStableIds: A list of stable Ids that identifies the association of  product structures to the
    session. This parameter is applicable only if parameter' productStructure' equals 'UseList'
    :var additionalPropsOnBOMWindow: A map('string,PropertyValues)' that specifies additional options that need to be
    set on the BOMWindow while returning the session product structure contents.
    """
    productStructure: str = ''
    outputFilters: StringMap = None
    relativeToSessionStableIds: List[str] = ()
    additionalPropsOnBOMWindow: PropertyValuesMap = None


@dataclass
class ProductStructureRecipe(TcBaseObj):
    """
    'ProductStructureRecipe' contains product, configuration and subset information.
    Note: During update of existing product structure information associated to the session the complete information
    like 'occurrenceLists', 'searchRecipe' and 'staticStructureFile' has to be passed
    
    :var structureContextIdentifier: The product and the configuration information.
    :var occurrenceLists: A list of filtered product structure results.
    :var searchRecipe: This is Fnd0SearchRecipe type BusinessObject which defines the search filters that were applied
    on the product structure
    :var staticStructureFile: The ImanFile object that represents the static product structure as expanded during the
    creation of session.
    """
    structureContextIdentifier: StructureContextIdentifier = None
    occurrenceLists: List[StructureSubsetInfo] = ()
    searchRecipe: BusinessObject = None
    staticStructureFile: ImanFile = None


@dataclass
class PropertyValues(TcBaseObj):
    """
    'PropertyValues' is a structure using which the property values that need to be set on objects are sent. Scalar
    property values are also sent via this structure.
    Note: In case to set a char property, populate the 'stringValues' as shown in example. Example
    To set a single char value say Y- provide 'stringValues[0][0]' = Y
    To set an array of char values YES - provide 'stringValues[0]' = YES
    
    :var intValues: List of integer values.
    :var boolValues: List of boolean values.
    :var dateValues: List of date values.
    :var stringValues: List of string values.
    :var floatValues: List of float values.
    :var doubleValues: List of double values.
    :var tagValues: List of object values.
    """
    intValues: List[int] = ()
    boolValues: List[bool] = ()
    dateValues: List[datetime] = ()
    stringValues: List[str] = ()
    floatValues: List[float] = ()
    doubleValues: List[float] = ()
    tagValues: List[BusinessObject] = ()


@dataclass
class RelatedObjectInfo(TcBaseObj):
    """
    'RelatedObjectInfo' structure contains an object associated to session, named references and reference objects.
    
    :var stableId: The identifier that enables the client to identify the association of the opened object with the
    session.
    :var relatedObject: The resulting related object by following a relation specified in the 'OpenSavedSessionFilter'.
    :var bomWindow: This is  only populated when the 'relatedObject' is associated with Fnd0AppSession with
    VisManagedDocument relation. The BOMWindow is constructed using 'fnd0Recipe' property on the relation.
    :var recipeObject: The recipe object of the structure which can be opened outside the context of the session.
    Typically it is the a VisStructureContext object but it could be any object which is capable of reconstructing the
    BOMWindow and hence the client should not assume the type of object to particular type. This is only populated when
    the 'relatedObject' is associated with Fnd0AppSession with VisManagedDocument relation and the relation has the
    'fnd0Recipe' property populated.
    :var relationProperties: A map('string,PropertyValues') on the relation that associated the session and the object.
    This has information about position, order, parent in case a document was inserted into another.
    Whe the the 'relatedObject' is associated with Fnd0AppSession with VisManagedDocument relation.
    The following information is returned.
    'fnd0InsertOrder' of type Integer - Indicates the order at which the 'relatedObject' was inserted into its parent.
    'fnd0ParentStableId' of type string - Indicates the parent object into whcih the 'relatedObject' was inserted.
    'fnd0Xform' of type string - Indicates the transformation of the  'relatedObject' in the co-ordinate system of
    parent.
    :var namedRefList: A List of named reference and reference object if the 'relatedObject' is a Dataset.
    """
    stableId: str = ''
    relatedObject: BusinessObject = None
    bomWindow: BOMWindow = None
    recipeObject: BusinessObject = None
    relationProperties: PropertyValuesMap = None
    namedRefList: List[NamedReferenceInfo] = ()


@dataclass
class RelatedObjectTypeAndNamedRefs(TcBaseObj):
    """
    This structure contains a related object type and the list of its named references to be processed.
    
    :var objectTypeName: Secondary object type based on which the results are to be filtered down.
    :var namedReferenceNames: Name of the named references in case the 'objectTypeName' is of Dataset type.
    """
    objectTypeName: str = ''
    namedReferenceNames: List[str] = ()


@dataclass
class RelationAndTypesFilter(TcBaseObj):
    """
    This structure contains a relation name and a list of related object types and its named references
    ('RelatedObjectTypeAndNamedRefs'). Together they are used to filter objects passed back in a session open operation.
    
    :var stableId: Stable ID that associates the object with the session.
    :var relationName: The name of the relation. Accepts a wild card "*" to consider all relation names. In case of
    'stableId' is provided this parameter is ignored.
    :var relatedObjAndNamedRefs: A list of related object types and named references.
    :var namedRefHandler: A string used to identify how named references will be processed. 
    Valid values are: 
    "'NoNamedRefs'" -- No named references are to be processed. The input '"relatedObjAndNamedRefs"' will be ignored. 
    "'AllNamedRefs'" -- All named references are to be processed. The input '"relatedObjAndNamedRefs"' will be ignored. 
    "'UseNamedRefsList'" -- Only the named references listed in the input '"relatedObjAndNamedRefs"' struct are
    processed.
    """
    stableId: str = ''
    relationName: str = ''
    relatedObjAndNamedRefs: List[RelatedObjectTypeAndNamedRefs] = ()
    namedRefHandler: str = ''


@dataclass
class RevisionRuleConfigInfo(TcBaseObj):
    """
    'RevisionRuleConfigInfo' is a structure which enables client to pass configuration information for RevisionRule and
    configuration parameters.
    
    :var revRule: The RevisionRule to be used for configuration.
    :var unitNo: Unit number. The client must pass -1 if unit number is not used for configuration.
    :var date: Effectivity date.
    :var today: Effectivity date to use today as the date.
    :var endItem: End Item.
    :var endItemRevision: End ItemRevision.
    :var overrideFolders: A list of Overriding Folder objects.
    """
    revRule: RevisionRule = None
    unitNo: int = 0
    date: datetime = None
    today: bool = False
    endItem: Item = None
    endItemRevision: ItemRevision = None
    overrideFolders: List[OverrideInfo] = ()


@dataclass
class BaseObjectAttachInfo(TcBaseObj):
    """
    'BaseObjectAttachInfo' is a structure which enables to create or update an object and attach it to the session. If
    relation is used to attach object to the session additional properties that are to be set on the relation can be
    passed as input.
    
    :var clientId: The input 'clientId' sent by the client application which will enable the client to map the output
    with the input.
    :var baseObjectToCreateOrUpdate: The input that has information to create an object or update an object associated
    to the session.
    :var relationName: The relation name used to associate the base object to the session. Typically, the ImanRelation
    name.
    :var additionalProps: A map ('string, PropertyValues') of property name and values pairs that need to be set on the
    ImanRelation specified via  'relationName'.
    Note: Incase the 'relationName' is VisManagedDocument, the 'clientId' of 'BaseObjectAttachInfo' can be passed as
    'fnd0ParentStableId' to indicate which document is the parent of this document, if this document was inserted into
    another in the session. The operation would first check if any 'BaseObjectAttachInfo' has the 'clientId' value same
    as the provided as 'fnd0ParentStableId' value, if yes then the 'stableId' of the 'BaseObjectAttachInfo' is stored
    as 'fnd0ParentStableId' otherwise the given string provided is stored as 'fnd0ParentStableId'.
    :var recipe: In case where the 'relationName' is VisManagedDocument, the structure configuration information that
    need to be set on 'fnd0Recipe' property of the relation.
    """
    clientId: str = ''
    baseObjectToCreateOrUpdate: CreateOrUpdateObjectInfo = None
    relationName: str = ''
    additionalProps: PropertyValuesMap = None
    recipe: ProductStructureRecipe = None


@dataclass
class BaseObjectAttachInfoOutput(TcBaseObj):
    """
    'BaseObjectAttachInfoOutput' holds information about the object that was associated to the session.
    
    :var clientId: The input 'clientId' sent by the client application which will enable the client to map the output
    with the input.
    :var stableId: The identifier that enables the client to identify the association of the opened object with the
    session.
    :var baseObject: The object that was associated to the session.
    """
    clientId: str = ''
    stableId: str = ''
    baseObject: BusinessObject = None


@dataclass
class SessionInfo(TcBaseObj):
    """
    'SessionInfo' is a structure which enables to create or update a session. The client can pass information to create
    a session or update a session by passing 'sessionToCreateOrUpdate' . During creation of session the client
    populates 'attachObjectsToSession', 'productSessionDataAttachInfos' with objects it has opened and populates
    'productAndConfigsToCreateOrUpdate' with product structure that were opened. The client can detach the closed
    documents and product structures by populating 'detachObjectOrProductsFromSession'. 
    Note: If an existing product structure which was associated to the session is opened in the context of the session,
    its configuration was changed or recipe was changed, and session is saved, the client application has to remove the
    original product and configuration from the session identifying it by 'stableId' and add the changes as new product
    structure configuration with same  or different 'stableId'.
    
    :var clientId: The input 'clientId' sent by the client application which will enable the client to map the output
    with the input.
    :var sessionToCreateOrUpdate: The input that has information to create a session BusinessObject. Since the session
    object is a Fnd0AppSession the required properties for object creation like 'object_name' and the BusinessObject
    type name can be passed.  Inorder to update, the object itself is passed.
    :var attachObjectsToSession: A list of information about the opened objects that are to be attached to the session.
    This would typically include any Dataset or Form that were opened in the application session.
    :var productAndConfigsToCreate: A list of information about the opened product structures that are to be attached
    to the session. This would typically include any BOMLine, VisStructureContext or Item, ItemRevision, BOMView,
    BOMViewRevision with configuration information.
    :var productAndConfigsToUpdate: A list of information about the product structures information attached to the
    session that are to be updated.
    Note: In order to update the configuration and search recipe the client has to remove the existing product
    structure information and re-associate the new product structure configuration.
    :var productSessionDataAttachInfos: A list of Fnd0ProductSessionData objects that are to be created or updated and
    attached to the session.
    :var detachObjectOrProductsFromSession: A list of stable Id of the product structure or objects associated with the
    session that must be detached.
    Note: This is considered only during the update case.
    """
    clientId: str = ''
    sessionToCreateOrUpdate: CreateOrUpdateObjectInfo = None
    attachObjectsToSession: List[BaseObjectAttachInfo] = ()
    productAndConfigsToCreate: List[StructureRecipeOfOpenedProduct] = ()
    productAndConfigsToUpdate: List[StructureRecipeRelatedInfo] = ()
    productSessionDataAttachInfos: List[ProductSessionDataAttachInfo] = ()
    detachObjectOrProductsFromSession: List[str] = ()


@dataclass
class SessionProductStructureInfo(TcBaseObj):
    """
    'SessionProductStructureInfo' holds information about the product structure associated to session.
    
    :var stableId: The identifier that enables the client to identify the association of the opened product structure
    with the session.
    :var bomWindow: The BOMWindow of the product structure that was associated to the session. This is populated only
    if  ProductStructureFilter.RecipeObjectsOnly is false.
    :var recipeObject: The recipe object of the structure which can be opened outside the context of the session.
    Typically it is the a VisStructureContext object but it could be any object which is capable of reconstructing the
    BOMWindow and hence the client should not assume the type of object to particular type.
    :var relationProperties: A map(string,PropertyValues) on the relation that associated the session  and the product
    structure . This has information about position, order and parent in case these properties were stored on the
    IMAN_CCContext relation. This map is only populated when the  IMAN_CCContext  relation that associates
    Fnd0AppSession and product structure has this information. 
    The following information is returned.
    'fnd0InsertOrder' of type Integer - Indicates the order at which the product structure was inserted into its parent.
    'fnd0ParentStableId' of type string - Indicates the parent product structure into which the product structure was
    inserted.
    'fnd0Xform' of type string - Indicates the transformation of the product structure in the co-ordinate system of
    parent
    :var staticStructureFileTicket: The static structure file ticket if the product structure was represented as a
    static file during the creation of session.
    Note: This is populated only if  'ProductStructureFilter.WantStaticStructureFileTicket' is true.
    :var relatedObjectInfos: A list of 'RelatedObjectInfo' that contains information about the objects that were
    related to the product structure information associated to session object.
    :var productSessionDataObjects: A list of Fnd0ProductSessionData that are associated to product structure
    information present in the session.
    """
    stableId: str = ''
    bomWindow: BOMWindow = None
    recipeObject: BusinessObject = None
    relationProperties: PropertyValuesMap = None
    staticStructureFileTicket: str = ''
    relatedObjectInfos: List[RelatedObjectInfo] = ()
    productSessionDataObjects: List[ProductSessionDataObjectInfo] = ()


@dataclass
class StructureContextIdentifier(TcBaseObj):
    """
    'StructureContextIdentifier' is a structure in which the application can provide information about the 'product'
    and its configurations that were opened. Using this structure an existing product and configuration is provided in
    the following forms. 
    - product: this could be an Item, ItemRevision, BOMView, BOMViewRevision or BOMLine or VisStructureContext or
    recipe object. If a BOMLine or VisStructureContext or any recipe object is passed then the configInfo need not be
    provided.
    
    
    
    :var product: This is either an Item, ItemRevision, BOMView, BOMViewRevision, BOMLine or StructureContext. The
    following is list the behaviour based on the type of object
    BOMLine - 'configInfo' need not be provided. If 'configInfo' is provided it will be ignored.
    Item or BOMView - The BOMWindow will be created using the ItemRevision by applying the 'configInfo' if provided
    otherwise by applying global RevisionRule on the Item.
    BOMViewRevision - The BOMWindow will be created using the ItemRevision of the BOMViewRevision.
    VisStructureContext  or any recipe object- If the object is already associated to the session nothing is done
    otherwise a copy of the recipe is created and associated to session.The 'configInfo' is ignored.
    :var bomViewType: The PSBOMView type that is to be used to create the BOMWindow. This is used only if Item or
    ItemRevision is supplied as product. If empty the default view type is used.
    :var configInfo: The configuration information for the product. This is used only if Item, ItemRevision, BOMView,
    BOMViewRevision are supplied as product.
    :var configOptions: A map ('string', 'PropertyValues') of property name and values pairs that need to be set on the
    BOMWindow.
    """
    product: BusinessObject = None
    bomViewType: BusinessObject = None
    configInfo: ConfigurationInfo = None
    configOptions: PropertyValuesMap = None


@dataclass
class StructurePathIdentifiers(TcBaseObj):
    """
    'StructurePathIdentifiers' is a structure describes the information that need to be used to traverse a BOMLine path.
    
    :var identifierPropertyName: The name of the property that is used to identify the BOMLine in the path.
    :var resultPathIdentifiers: A list of property values to match when the BOMLine path is traversed.
    """
    identifierPropertyName: str = ''
    resultPathIdentifiers: List[str] = ()


@dataclass
class StructureRecipeOfOpenedProduct(TcBaseObj):
    """
    'StructureRecipeOfOpenedProduct' is a structure in which the application can provide information about the product
    and its configurations that were opened. Using this structure, the subsets or product structures that were added,
    updated or removed to the existing product structure that is associated to session can be updated.
    
    :var clientId: The input 'clientId' sent by the client application which will enable the client to map the output
    with the input.
    :var objsAttachInfos: A list of information about the opened objects that are to be associated to the products
    opened in the context of the session. This would typically include any Dataset or Form that are to associated to
    the product structure in the context of session.
    :var productSessionDataAttachInfos: A list of Fnd0ProductSessionData objects that are to be created or updated and
    attached to the product structure in the context of the session.
    :var structureRecipe: Object of type 'ProductStructureRecipe' which contains product, configuration and subset
    information.
    :var structureRecipeProps: A map ('string', 'PropertyValues') of property name and values pairs that need to be set
     or updated on the ImanRelation that associates the product structure to the session.
    Note':' If the client would like to re-use the same stableId of a detached product structure, then it can be
    accomplished by passing the stable id as value for 'fnd0CopyStableId' property.
    """
    clientId: str = ''
    objsAttachInfos: List[BaseObjectAttachInfo] = ()
    productSessionDataAttachInfos: List[ProductSessionDataAttachInfo] = ()
    structureRecipe: ProductStructureRecipe = None
    structureRecipeProps: PropertyValuesMap = None


@dataclass
class StructureRecipeOfProductOutput(TcBaseObj):
    """
    'StructureRecipeOfOpenedOutput' holds information about the product structure that was associated to the session.
    
    :var clientId: The input 'clientId' sent by the client application which will enable the client to map the output
    with the input.
    :var relativeToSessionStableId: The identifier that enables the client to identify the association of the opened
    product structure with the session.
    :var recipeObject: The recipe object which can be opened outside the context of the session. Typically it&rsquo;s a
    VisStructureContext object but the client application should not assume the type of the object. The type of the
    object returned could be changed in the future. The object is solely returned so that the object can be passed to
    operations that supports creation of BOMWindow using this object.
    :var baseObjectAttachOutputs: A list of 'BaseObjectAttachInfoOutput' which has information about the objects that
    were associated to product structure information in the session.
    :var productSessionDataAttachOutputs: A list of Fnd0ProductSessionData objects that were created or updated and
    attached to the product structure information in the session.
    """
    clientId: str = ''
    relativeToSessionStableId: str = ''
    recipeObject: BusinessObject = None
    baseObjectAttachOutputs: List[BaseObjectAttachInfoOutput] = ()
    productSessionDataAttachOutputs: List[BaseObjectAttachInfoOutput] = ()


@dataclass
class StructureRecipeRelatedInfo(TcBaseObj):
    """
    'StructureRecipeRelatedInfo' contains the product structure information that is associated to the session which is
    to be updated.
    
    :var stableId: The stableId of existing product structure information.
    :var objsAttachInfos: A list of information about the opened objects that are to be associated to the product
    structures opened in the context of the session. This would typically include any Dataset or Form that are to
    associated to the product structure in the context of session.
    :var productSessionDataAttachInfos: A list of Fnd0ProductSessionData objects that are to be created or updated and
    attached to the product structure information in the context of the session.
    :var detachObjects: A list of stable Id of the objects associated with the product structure information in the
    context of the session that must be detached.
    :var structureRecipeProps: A map ('string, PropertyValues') of property name and values pairs that need to be set
    or updated on the ImanRelation that associates the product structure to the session.
    """
    stableId: str = ''
    objsAttachInfos: List[BaseObjectAttachInfo] = ()
    productSessionDataAttachInfos: List[ProductSessionDataAttachInfo] = ()
    detachObjects: List[str] = ()
    structureRecipeProps: PropertyValuesMap = None


@dataclass
class StructureSubsetInfo(TcBaseObj):
    """
    'StructureSubsetInfo' is a structure that has the results of product structure filter which needs to be saved in
    the session.
    
    :var resultContext: Context of the 'resultObjects'. This can be one of following values 
    - 'fnd0bg_occurrences
    - fnd0sel_bg_occurrences
    - fnd0sel_target_occurrences
    - fnd0target_occurrences'
    - 'occurrence_list'
    
    
    If empty default value is assumed as 'occurrence'_lis't'
    :var resultObjects: Results of the product structure filter.
    """
    resultContext: str = ''
    resultObjects: SubsetLinesInfo = None


@dataclass
class SubsetLinesInfo(TcBaseObj):
    """
    'SubsetLinesInfo' is a structure using which the result lines of a search can be persisted along with the subset.
    The lines could be identified as BOMLine object or using an identifier.
    
    :var lines: Result BOMLine that are part of the subset.
    :var compositePathIdentifiers: Result BOMLine identified by path identifiers.
    """
    lines: List[BusinessObject] = ()
    compositePathIdentifiers: List[CompositePathIdentifiers] = ()


@dataclass
class CompositePathIdentifiers(TcBaseObj):
    """
    'CompositePathIdentifiers' is a structure in which the application can provide information about how the BOMLine
    can be identified using identifiers. The identifiers can be of composite in nature which means traverse the BOMLine
    path up to certain level using a particular BOMLine property and then the rest of the path using different
    property. For example, with input like:
    
    'pathIdentifiers[0].identifierPropertyName = "bl_topline_absocc_id"
    pathIdentifiers[0]. resultPathIdentifiers[0] = "absOccUId"
    pathIdentifiers[1].identifierPropertyName = "bl_clone_stable_occurrence_id"
    pathIdentifiers[1].resultPathIdentifiers[0] = "ABC"
    pathIdentifiers[1].resultPathIdentifiers[1] = "CAD"
    pathIdentifiers[1].resultPathIdentifiers[2] = "XYZ"'
    
    While traversing from top BOMLine, the BOMLine that has the property bl_topline_absocc_id=absOccUId will be
    searched followed by the property  'bl_clone_stable_occurrence_id' for traversing the hierarchy. Currently only '
    bl_clone_stable_occurrence_id' is supported to identify the entire BOMLine path.
    
    :var pathIdentifiers: A list of identifiers that identifies the BOMLine in a structure.
    """
    pathIdentifiers: List[StructurePathIdentifiers] = ()


@dataclass
class ConfigurationInfo(TcBaseObj):
    """
    'ConfigurationInfo' is a structure which enables client to pass configuration information for product structures.
    
    :var revRuleConfigInfo: The RevisionRule and the configuration entry parameters.
    :var variantRulesOrOptionSets: A list of VariantRule or StoredOptionSet to be used for configuration.
    :var activeAssemblyArrangement: AssemblyArrangement to be used for configuration.
    :var configContext: Configuration object to be used for configuration. If used then 'revRuleConfigInfo',
    'variantRulesOrOptionSets' and 'activeAssemblyArrangement' are ignored.
    :var bomWinPropFlagMap: A map ('string', 'PropertyValues') of property name and values pairs that need to be set on
    the BOMWindow.
    :var effGrpRevList: A list of Fnd0EffectivityGrp objects, 'effGrpRevList' is used along with BOMWindow  to
    configure.
    """
    revRuleConfigInfo: RevisionRuleConfigInfo = None
    variantRulesOrOptionSets: List[BusinessObject] = ()
    activeAssemblyArrangement: AssemblyArrangement = None
    configContext: BusinessObject = None
    bomWinPropFlagMap: PropertyValuesMap = None
    effGrpRevList: List[ItemRevision] = ()


@dataclass
class CreateInput(TcBaseObj):
    """
    'CreateInput' is a structure used to capture the inputs required for creation of a business object. This is a
    recursive structure containing the 'CreateInput'(s) for any secondary (compounded) objects that might be created
    (e.g. Item also creates ItemRevision and ItemMasterForm).
    
    :var boName: Business Object name.
    :var propertyNameValues: A map ('string, PropertyValues') of property name and values pairs that need to be part of
    create input.
    :var compoundCreateInput: Compounded create input in case the business object needs secondary object to be created.
    """
    boName: str = ''
    propertyNameValues: PropertyValuesMap = None
    compoundCreateInput: CreateInputMap = None


@dataclass
class CreateObjectInfo(TcBaseObj):
    """
    'CreateObjectInfo' is a structure which provides input to create an object.
    
    :var creInp: The 'CreateInput' to create the object.
    :var relationName: The relation to be used to associate the created object to the 'relateToObject'. If this is not
    provided and 'relateToObject' is provided then the relation type is determined using the preference
    <Primary>_<Secondary>_default_relation. For example, if relateToObject is of type ItemRevision and boName is
    DirectModel then preference 
    ItemRevision_DirectModel_default_relation used. If not found, then ItemRevision_default_relation will be searched
    for and used.
    :var relateToObject: The object to which the created object is to be related with the given 'relationName'. 
    Special cases -
    1. Folder object without 'relationName' is passed then the created object is added as a folder content.
    2. BOMLine is passed then the system creates an attachment line.
    """
    creInp: CreateInput = None
    relationName: str = ''
    relateToObject: BusinessObject = None


@dataclass
class CreateOrUpdateObjectInfo(TcBaseObj):
    """
    'CreateOrUpdateObjectInfo' is a structure which provides input to either create or update an object.
    'Note:' If 'objectToUpdate' is provided then 'objectToCreate' will be ignored.
    
    :var objectToCreate: The 'CreateInput' to create the object. In case a Dataset is requested to be created then the
    default values for' tool_used', 'dataset_type' and 'format_used' will be assumed.
    :var objectToUpdate: The object to be updated.
    :var lmdOfObject: Last modified date of the input object. If this input date is different than the current last
    modified date and the 'overwriteForLastModDate' preference is false, the input will be ignored, and processing will
    continue with the next input. In this scenario, error 215033 will be returned. If the dates are different and the
    'overwriteForLastModDate' preference is true, processing will continue with the current input. In this scenario,
    error 215034 will be returned.
    :var overwriteForLastModDate: Flag to check whether the object needs to be updated if the input last modified date
    is different from the current last modified date of the object in Teamcenter.
    :var objectPropsToUpdate: A map ('string', 'PropertyValues') of property name and values pairs that need to be set
    on the created or updated object.
    """
    objectToCreate: CreateObjectInfo = None
    objectToUpdate: BusinessObject = None
    lmdOfObject: datetime = None
    overwriteForLastModDate: bool = False
    objectPropsToUpdate: PropertyValuesMap = None


@dataclass
class CreateOrUpdateSessionOutput(TcBaseObj):
    """
    'CreateOrUpdateSessionOutput' contains the session object and a list of objects and product structures that were
    associated to the session.
    
    :var clientId: The input 'clientId' sent by the client application which will enable the client to map the output
    with the input.
    :var sessionObject: Fnd0AppSession that represents the session.
    :var baseObjectAttachOutputs: A list of 'BaseObjectAttachInfoOutput' which has information about the objects that
    were associated to session.
    :var recipeOfOpenedProductOutputs: A list of 'StructureRecipeOfProductOutput' that contains information about the
    product structures that were associated to session.
    :var productSessionDataAttachOutputs: A list of Fnd0ProductSessionData objects that were created or updated and
    attached to the session object.
    """
    clientId: str = ''
    sessionObject: BusinessObject = None
    baseObjectAttachOutputs: List[BaseObjectAttachInfoOutput] = ()
    recipeOfOpenedProductOutputs: List[StructureRecipeOfProductOutput] = ()
    productSessionDataAttachOutputs: List[BaseObjectAttachInfoOutput] = ()


@dataclass
class CreateOrUpdateSessionResponse(TcBaseObj):
    """
    'CreateOrUpdateSessionResponse' contains a list of 'CreateOrUpdateSessionOutput'
    Each of which has the session object along with it are the objects and product structures that were associated to
    it. Stable Id that identifies the associated objects is also returned which could be used during opening the
    session or referencing them from within the CAD files.
    
    :var sessionOutputs: A list of objects that were associated to the session.
    :var serviceData: 'ServiceData' that contains the partial errors if any.
    """
    sessionOutputs: List[CreateOrUpdateSessionOutput] = ()
    serviceData: ServiceData = None


"""
'Map'('string', 'PropertyValues') of property name to property values.
"""
PropertyValuesMap = Dict[str, PropertyValues]


"""
'StringMap'(string, string) is a map of string to string.
"""
StringMap = Dict[str, str]


"""
'CreateInputMap'('string', list of 'CreateInput') is a map of reference or relation property name to secondary 'CreateInput' objects.
"""
CreateInputMap = Dict[str, List[CreateInput]]



from tcsoa.gen.Workflow._2008_06.services import WorkflowService as imp0
from tcsoa.gen.Workflow._2015_07.services import WorkflowService as imp1
from tcsoa.gen.Workflow._2010_09.services import WorkflowService as imp2
from tcsoa.gen.Workflow._2019_06.services import WorkflowService as imp3
from tcsoa.gen.Workflow._2014_10.services import WorkflowService as imp4
from tcsoa.gen.Workflow._2020_01.services import WorkflowService as imp5
from tcsoa.gen.Workflow._2013_05.services import WorkflowService as imp6
from tcsoa.gen.Workflow._2012_10.services import WorkflowService as imp7
from tcsoa.gen.Workflow._2014_06.services import WorkflowService as imp8
from tcsoa.gen.Workflow._2007_06.services import WorkflowService as imp9
from tcsoa.utils import TcService


class WorkflowService(TcService):
    addAttachments = imp0.addAttachments
    addSignoffs = imp0.addSignoffs
    addSignoffs2 = imp1.addSignoffs
    applyTemplateToProcesses = imp2.applyTemplateToProcesses
    applyTemplateToProcessesAsync = imp2.applyTemplateToProcessesAsync
    assignAllTasks = imp0.assignAllTasks
    changeState = imp0.changeState
    createInstance = imp0.createInstance
    createOrUpdateHandler = imp3.createOrUpdateHandler
    createOrUpdatePAL = imp3.createOrUpdatePAL
    createOrUpdateTemplate = imp3.createOrUpdateTemplate
    createRemoteWorkflowAsync = imp4.createRemoteWorkflowAsync
    createWorkflow = imp4.createWorkflow
    delegateSignoff = imp0.delegateSignoff
    getAllTasks = imp0.getAllTasks
    getAssignmentLists = imp0.getAssignmentLists
    getRegisteredHandlers = imp3.getRegisteredHandlers
    getResourcePool = imp0.getResourcePool
    getSupportedHandlerArguments = imp5.getSupportedHandlerArguments
    getWorkflowTemplates = imp0.getWorkflowTemplates
    getWorkflowTemplates2 = imp6.getWorkflowTemplates
    listDefinitions = imp0.listDefinitions
    performAction = imp0.performAction
    performAction2 = imp7.performAction2
    performAction3 = imp8.performAction3
    performActionWithSignature = imp8.performActionWithSignature
    removeAttachments = imp0.removeAttachments
    removeSignoffs = imp0.removeSignoffs
    setActiveSurrogate = imp8.setActiveSurrogate
    setOutOfOffice = imp8.setOutOfOffice
    setReleaseStatus = imp9.setReleaseStatus
    setSurrogate = imp8.setSurrogate
    viewAuditFile = imp0.viewAuditFile



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from tcsoa.gen.Manufacturing._2017_11.DataManagement import GetConnectorResponse, GetPhysicalAttachmentsInput, SetConnectorInput, GetPhysicalAttachmentsResponse, SetPhysicalAttachmentsInput, RemovePhysicalAttachmentRelInput
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcService


class DataManagementService(TcService):

    @classmethod
    def removePhysicalAttachementRelation(cls, input: List[RemovePhysicalAttachmentRelInput]) -> ServiceData:
        """
        This operation deletes Mfg0MEPhysicalAttachment or Mfg0MEMountToolToRobot relation between the AbsOccurrence of
        given source BOMLine and target BOMLine objects for given scope Mfg0BvrWorkarea and context Mfg0BvrWorkarea
        acting as a root of the structure.
        
        Use cases:
        Line Designer user wants to disconnect two connected resource object (for e.g. Mfg0MEFactoryTool from the
        Mfg0MERobot ) which are connected with Mfg0MEPhysicalAttachment or Mfg0MEMountToolToRobot relation in a given
        scope Mfg0BvrWorkarea and context Mfg0BvrWorkarea acting as a root of the structure.
        """
        return cls.execute_soa_method(
            method_name='removePhysicalAttachementRelation',
            library='Manufacturing',
            service_date='2017_11',
            service_name='DataManagement',
            params={'input': input},
            response_cls=ServiceData,
        )

    @classmethod
    def setConnectorInfo(cls, input: List[SetConnectorInput]) -> ServiceData:
        """
        Connector is Product and Manufacturing Information (PMI) object created by NX which is used to define the
        connection between two components used on shop floor. This operation sets the connector information stored as
        Mfg0MEConnectorTableRow in Mfg0MEConnectorTable. The Mfg0MEConnectorTableForm is attached to the ItemRevision. 
        If the Mfg0MEConnectorTableForm is not related to the ItemRevision the operation first creates
        Mfg0MEConnectorTableForm and attaches it to the given ItemRevision with a relation Mfg0MEConnectorTblFormRel.
        If given input connector id is not present in Mfg0MEConnectorTableRow, then a Mfg0MEConnectorTableRow is added
        in Mfg0MEConnectorTable with information connector type, connector name, connector ID and transformation. 
        If given input connector id is present in Mfg0MEConnectorTableRow,the row is updated with latest information. 
        All the Mfg0MEConnectorTableRow for which information is not given are removed from Mfg0MEConnectorTable.
        
        Use cases:
        Line Designer user wants to add, remove or update the connector information for ItemRevision
        """
        return cls.execute_soa_method(
            method_name='setConnectorInfo',
            library='Manufacturing',
            service_date='2017_11',
            service_name='DataManagement',
            params={'input': input},
            response_cls=ServiceData,
        )

    @classmethod
    def setPhysicalAttachementsInScope(cls, input: List[SetPhysicalAttachmentsInput]) -> ServiceData:
        """
        This operation creates physical attachment Mfg0MEPhysicalAttachment or  Mfg0MEMountToolToRobot relation between
        the AbsOccurrence of given source BOMLine and target BOMLine objects for given scope Mfg0BvrWorkarea and
        context Mfg0BvrWorkarea acting as a root of the structure.
        
        Use cases:
        Line Designer user wants to set mount and attach information for BOMLine (e.g. Mfg0MEFactoryTool and
        Mfg0MERobot) using  Mfg0MEPhysicalAttachment or Mfg0MEMountToolToRobot relation in a given scope
        Mfg0BvrWorkarea and context Mfg0BvrWorkarea acting as a root of the structure.
        """
        return cls.execute_soa_method(
            method_name='setPhysicalAttachementsInScope',
            library='Manufacturing',
            service_date='2017_11',
            service_name='DataManagement',
            params={'input': input},
            response_cls=ServiceData,
        )

    @classmethod
    def getConnectorInfo(cls, itemRevs: List[BusinessObject]) -> GetConnectorResponse:
        """
        Connector is Product and Manufacturing Information (PMI) object created by NX which is used to define the
        connection between two components used on shop floor. This operation retrieves the information of connectors
        represented as a Mfg0MEConnectorTableRow in Mfg0MEConnectorTable. 
        Mfg0MEConnectorTableRow has information about connector type, connector name, connector ID and transformation.
        The Mfg0MEConnectorTableForm holds Mfg0MEConnectorTable and is related to ItemRevision through the relation
        Mfg0MEConnectorTblFormRel.
        
        Use cases:
        Connector is Product and Manufacturing Information (PMI) object created by NX which is used to define the
        connection between two components used on shop floor. E.g. Mfg0Conveyor and Mfg0Conveyor or Mfg0Conveyor and
        Mfg0MERobot. Line Designer user wants to retrieve connector information for ItemRevision.
        """
        return cls.execute_soa_method(
            method_name='getConnectorInfo',
            library='Manufacturing',
            service_date='2017_11',
            service_name='DataManagement',
            params={'itemRevs': itemRevs},
            response_cls=GetConnectorResponse,
        )

    @classmethod
    def getPhysicalAttachmentsInScope(cls, input: List[GetPhysicalAttachmentsInput]) -> GetPhysicalAttachmentsResponse:
        """
        This operation retrievs all physical attachments (Mfg0MEPhysicalAttachment or Mfg0MEMountToolToRobot) relations
        defined between two BOMLine objects that are children of the given scope Mfg0BvrWorkarea and context
        Mfg0BvrWorkarea acting as a root of the structure.
        This operation
        - Processes the input scope Mfg0BvrWorkarea under root context Mfg0BvrWorkarea in Bill of Equipment structure.
        - Traverses   the scope, finds the AbsOccurrence under the scope related with Mfg0MEPhysicalAttachment or
        Mfg0MEMountToolToRobot relation. From primary AbsOccurrence of relation Mfg0MEPhysicalAttachment or
        Mfg0MEMountToolToRobot it collects source BOMLine and from secondary AbsOccurrence it collects target BOMLine
        along with the relation properties on Mfg0MEPhysicalAttachment or Mfg0MEMountToolToRobot.
        
        
        
        Use cases:
        Line Designer user wants to retrieve mount and attachment information for the BOMLine connections with
        Mfg0MEPhysicalAttachment relation in a given scope of Mfg0BvrWorkarea and Mfg0BvrWorkarea acting as a root of
        the structure.
        """
        return cls.execute_soa_method(
            method_name='getPhysicalAttachmentsInScope',
            library='Manufacturing',
            service_date='2017_11',
            service_name='DataManagement',
            params={'input': input},
            response_cls=GetPhysicalAttachmentsResponse,
        )



from __future__ import annotations

from tcsoa.gen.Internal.IcsAw._2018_12.Classification import ClassificationObjectInfo2, FindClassificationInfo2Response, SaveClassificationObjects2Response
from typing import List
from tcsoa.gen.Internal.IcsAw._2018_05.Classification import SearchCriteria
from tcsoa.utils import TcService
from tcsoa.gen.BusinessObjects import WorkspaceObject


class ClassificationService(TcService):

    @classmethod
    def findClassificationInfo2(cls, workspaceObjects: List[WorkspaceObject], searchCriterias: List[SearchCriteria], classificationDataOptions: int) -> FindClassificationInfo2Response:
        """
        Finds classification information based on the input criteria.
        
        A populated list If a list of WorkspaceObject objects are provided  ; then this operation finds all the
        classification objects for the given WorkspaceObject.
        In caseIf a given WorkspaceObject is not classified this operation finds and returns standalone classification
        object; which could then be linked to given WorkspaceObject.
        If a given workspace object is not classified and also the system does not contain standalone classification
        object which could be linked to the WorkspaceObject then this operation will return classification hierarchy
        which could be used for browsing and classifying an object in a classification class of interest. (For
        traditional classification system, children of "Classification Root" (ICM) are returned. For next generation
        Classification Standard Taxonomy (CST) system, children of the Default Hierarchy are returned).
        Empty list of  WorkspaceObject would be passed in if user wants to search for a class in classification
        hierarchy using search criteria.
        This operations takes care of returning classification node information from next generation classification
        system including, containing cardinal and polymorphic blocks.
        
        If a search criteria is provided; then this operation searches for classification classes in classification
        hierarchy and returns their metadata information like class ID, name, class parents etc. If both
        searchCriterias and workspaceObjects are provided, workspaceObjects list is ignored and search criteria is used.
        
        Use cases:
        This operation could be used when user needs to find classification objects, for both traditional and
        Classification Standard Taxonomy (CST), based on workspace objects. Each time a WorkspaceObject is classified
        in a classification class a classification object is created. After searching for all the classification
        objects corresponding to a WorkspaceObject, user can find more information about the classification(s) of a
        WorkspaceObject. 
        The operation findClassificationInfo2 can be used to get detailed information about the classification objects,
        classification classes where these classification objects belong. After getting information about
        classification objects the user can choose to modify or delete the objects by using operation:
        saveClassificationObjects2 or deleteClassificationObjects, both of which support traditional and Classification
        Standard Taxonomy (CST) classes.
        
        The operation findClassificationInfo2 can also be used when a user wants to search for classes in
        classification hierarchy by using any of class' properties. After searching for classification classes; the
        user can use that information to browse through a subset of classification hierarchy and classify a
        classifiable workspace object in a classification class of interest.
        """
        return cls.execute_soa_method(
            method_name='findClassificationInfo2',
            library='Internal-IcsAw',
            service_date='2018_12',
            service_name='Classification',
            params={'workspaceObjects': workspaceObjects, 'searchCriterias': searchCriterias, 'classificationDataOptions': classificationDataOptions},
            response_cls=FindClassificationInfo2Response,
        )

    @classmethod
    def saveClassificationObjects2(cls, classificationObjects: List[ClassificationObjectInfo2]) -> SaveClassificationObjects2Response:
        """
        Creates or updates one or more classification objects and (optionally) attach them to a WorkspaceObject object,
        thus classifying it. When the Classification objects are not associated with any WorkspaceObject they act as
        standalone classification objects. A classification object is also called an ICO object. This supports
        traditional icm0, next generation Cls0Object, and Classification Standard Taxonomy (CST) Cls0CstObject.
        This operation can also be used to create or update classification objects in next generation classification
        hierarchy containing cardinal and polymorphic blocks.
        
        Use cases:
        User wants to classify a WorkspaceObject or create a standalone classification object (ICO) or update an
        existing classification object in Active Workspace, in either a traditional or Classification Standard Taxonomy
        (CST) class. Additionally, the user could be performing a delta update on a CST Property Record. This operation
        can be combined with other operations like createItems() to create workspace object and then associate the
        workspace object to the classification object. Before creating a classification object, a classification class
        hierarchy should already be created by the classification admin user in Teamcenter. This hierarchy should
        include a storage class (a class that allows instances to be created and associated to it) for which the
        classification objects need to be created. Values of any attributes associated with classification objects can
        also be populated.
        """
        return cls.execute_soa_method(
            method_name='saveClassificationObjects2',
            library='Internal-IcsAw',
            service_date='2018_12',
            service_name='Classification',
            params={'classificationObjects': classificationObjects},
            response_cls=SaveClassificationObjects2Response,
        )
